'use strict';

var pluginCatalogNode = require('@backstage/plugin-catalog-node');
var clientOrganizations = require('@aws-sdk/client-organizations');
var config = require('../awsOrganization/config.cjs.js');
var integrationAwsNode = require('@backstage/integration-aws-node');

const AWS_ORGANIZATION_REGION = "us-east-1";
const LOCATION_TYPE = "aws-cloud-accounts";
const ACCOUNTID_ANNOTATION = "amazonaws.com/account-id";
const ACCOUNT_EMAIL_ANNOTATION = "amazonaws.com/account-email";
const ARN_ANNOTATION = "amazonaws.com/arn";
const ORGANIZATION_ANNOTATION = "amazonaws.com/organization-id";
const ACCOUNT_STATUS_LABEL = "amazonaws.com/account-status";
class AwsOrganizationCloudAccountProcessor {
  constructor(credProvider, logger) {
    this.credProvider = credProvider;
    this.logger = logger?.child({
      target: this.getProcessorName()
    });
    this.organizations = new clientOrganizations.Organizations({
      credentialDefaultProvider: () => this.credProvider.sdkCredentialProvider,
      region: AWS_ORGANIZATION_REGION
    });
  }
  organizations;
  logger;
  static async fromConfig(config$1, options) {
    const c = config$1.getOptionalConfig("catalog.processors.awsOrganization");
    const orgConfig = c ? config.readAwsOrganizationConfig(c) : void 0;
    if (orgConfig?.roleArn) {
      options.logger.warn(
        "The roleArn configuration for AwsOrganizationCloudAccountProcessor ignores the role name, use accountId configuration instead"
      );
    }
    const awsCredentialsManager = integrationAwsNode.DefaultAwsCredentialsManager.fromConfig(config$1);
    const credProvider = await awsCredentialsManager.getCredentialProvider({
      arn: orgConfig?.roleArn,
      accountId: orgConfig?.accountId
    });
    return new AwsOrganizationCloudAccountProcessor(
      credProvider,
      options.logger
    );
  }
  getProcessorName() {
    return "AwsOrganizationCloudAccountProcessor";
  }
  async readLocation(location, _optional, emit) {
    if (location.type !== LOCATION_TYPE) {
      return false;
    }
    this.logger?.info("Discovering AWS Organization Account objects");
    (await this.getAwsAccounts()).map((account) => this.mapAccountToComponent(account)).filter((entity) => {
      if (location.target !== "") {
        if (entity.metadata.annotations) {
          return entity.metadata.annotations[ORGANIZATION_ANNOTATION] === location.target;
        }
        return false;
      }
      return true;
    }).forEach((entity) => {
      emit(pluginCatalogNode.processingResult.entity(location, entity));
    });
    return true;
  }
  normalizeName(name) {
    return name.trim().toLocaleLowerCase("en-US").replace(/[^a-zA-Z0-9\-]/g, "-");
  }
  normalizeAccountStatus(name) {
    return name.toLocaleLowerCase("en-US");
  }
  extractInformationFromArn(arn) {
    const parts = arn.split("/");
    return {
      accountId: parts[parts.length - 1],
      organizationId: parts[parts.length - 2]
    };
  }
  async getAwsAccounts() {
    let awsAccounts = [];
    let isInitialAttempt = true;
    let nextToken = void 0;
    while (isInitialAttempt || nextToken) {
      isInitialAttempt = false;
      const orgAccounts = await this.organizations.listAccounts({ NextToken: nextToken });
      if (orgAccounts.Accounts) {
        awsAccounts = awsAccounts.concat(orgAccounts.Accounts);
      }
      nextToken = orgAccounts.NextToken;
    }
    return awsAccounts;
  }
  mapAccountToComponent(account) {
    const { accountId, organizationId } = this.extractInformationFromArn(
      account.Arn
    );
    return {
      apiVersion: "backstage.io/v1alpha1",
      kind: "Resource",
      metadata: {
        annotations: {
          [ACCOUNTID_ANNOTATION]: accountId,
          [ARN_ANNOTATION]: account.Arn || "",
          [ORGANIZATION_ANNOTATION]: organizationId,
          [ACCOUNT_EMAIL_ANNOTATION]: account.Email || ""
        },
        labels: {
          [ACCOUNT_STATUS_LABEL]: this.normalizeAccountStatus(
            account.Status || ""
          )
        },
        name: this.normalizeName(account.Name || ""),
        title: account.Name || "",
        namespace: "default"
      },
      spec: {
        type: "cloud-account",
        owner: "unknown"
      }
    };
  }
}

exports.AwsOrganizationCloudAccountProcessor = AwsOrganizationCloudAccountProcessor;
//# sourceMappingURL=AwsOrganizationCloudAccountProcessor.cjs.js.map
