'use strict';

var integration = require('@backstage/integration');
var pluginCatalogNode = require('@backstage/plugin-catalog-node');
var config = require('./config.cjs.js');
var clientS3 = require('@aws-sdk/client-s3');
var uuid = require('uuid');
var middlewareEndpoint = require('@aws-sdk/middleware-endpoint');
var integrationAwsNode = require('@backstage/integration-aws-node');

function _interopNamespaceCompat(e) {
  if (e && typeof e === 'object' && 'default' in e) return e;
  var n = Object.create(null);
  if (e) {
    Object.keys(e).forEach(function (k) {
      if (k !== 'default') {
        var d = Object.getOwnPropertyDescriptor(e, k);
        Object.defineProperty(n, k, d.get ? d : {
          enumerable: true,
          get: function () { return e[k]; }
        });
      }
    });
  }
  n.default = e;
  return Object.freeze(n);
}

var uuid__namespace = /*#__PURE__*/_interopNamespaceCompat(uuid);

class AwsS3EntityProvider {
  constructor(config, integration, awsCredentialsManager, logger, taskRunner) {
    this.config = config;
    this.integration = integration;
    this.awsCredentialsManager = awsCredentialsManager;
    this.logger = logger.child({
      target: this.getProviderName()
    });
    this.scheduleFn = this.createScheduleFn(taskRunner);
  }
  logger;
  s3;
  scheduleFn;
  connection;
  endpoint;
  static fromConfig(configRoot, options) {
    const providerConfigs = config.readAwsS3Configs(configRoot);
    const integration$1 = integration.ScmIntegrations.fromConfig(configRoot).awsS3.list()[0];
    if (!integration$1) {
      throw new Error("No integration found for awsS3");
    }
    if (!options.schedule && !options.scheduler) {
      throw new Error("Either schedule or scheduler must be provided.");
    }
    return providerConfigs.map((providerConfig) => {
      if (!options.schedule && !providerConfig.schedule) {
        throw new Error(
          `No schedule provided neither via code nor config for awsS3-provider:${providerConfig.id}.`
        );
      }
      const awsCredentialsManager = integrationAwsNode.DefaultAwsCredentialsManager.fromConfig(configRoot);
      const taskRunner = options.schedule ?? options.scheduler.createScheduledTaskRunner(providerConfig.schedule);
      return new AwsS3EntityProvider(
        providerConfig,
        integration$1,
        awsCredentialsManager,
        options.logger,
        taskRunner
      );
    });
  }
  createScheduleFn(taskRunner) {
    return async () => {
      const taskId = `${this.getProviderName()}:refresh`;
      return taskRunner.run({
        id: taskId,
        fn: async () => {
          const logger = this.logger.child({
            class: AwsS3EntityProvider.prototype.constructor.name,
            taskId,
            taskInstanceId: uuid__namespace.v4()
          });
          try {
            await this.refresh(logger);
          } catch (error) {
            logger.error(
              `${this.getProviderName()} refresh failed, ${error}`,
              error
            );
          }
        }
      });
    };
  }
  /** {@inheritdoc @backstage/plugin-catalog-node#EntityProvider.getProviderName} */
  getProviderName() {
    return `awsS3-provider:${this.config.id}`;
  }
  /** {@inheritdoc @backstage/plugin-catalog-node#EntityProvider.connect} */
  async connect(connection) {
    this.connection = connection;
    const { accountId, region, bucketName } = this.config;
    const credProvider = await this.awsCredentialsManager.getCredentialProvider(
      accountId ? { accountId } : void 0
    );
    this.s3 = new clientS3.S3({
      customUserAgent: "backstage-aws-catalog-s3-entity-provider",
      apiVersion: "2006-03-01",
      credentialDefaultProvider: () => credProvider.sdkCredentialProvider,
      endpoint: this.integration.config.endpoint,
      region,
      forcePathStyle: this.integration.config.s3ForcePathStyle
    });
    const endpoint = await middlewareEndpoint.getEndpointFromInstructions(
      {
        Bucket: bucketName
      },
      clientS3.ListObjectsV2Command,
      this.s3.config
    );
    if (endpoint?.url)
      this.endpoint = endpoint.url.href.endsWith("/") ? endpoint.url.href : `${endpoint.url.href}/`;
    await this.scheduleFn();
  }
  async refresh(logger) {
    if (!this.connection) {
      throw new Error("Not initialized");
    }
    logger.info("Discovering AWS S3 objects");
    const keys = await this.listAllObjectKeys();
    logger.info(`Discovered ${keys.length} AWS S3 objects`);
    const locations = keys.map((key) => this.createLocationSpec(key));
    await this.connection.applyMutation({
      type: "full",
      entities: locations.map((location) => {
        return {
          locationKey: this.getProviderName(),
          entity: pluginCatalogNode.locationSpecToLocationEntity({ location })
        };
      })
    });
    logger.info(`Committed ${locations.length} Locations for AWS S3 objects`);
  }
  async listAllObjectKeys() {
    if (!this.s3) {
      throw new Error("Not initialized");
    }
    const keys = [];
    let continuationToken = void 0;
    let output;
    do {
      output = await this.s3.listObjectsV2({
        Bucket: this.config.bucketName,
        ContinuationToken: continuationToken,
        Prefix: this.config.prefix
      });
      if (output.Contents) {
        output.Contents.forEach((item) => {
          if (item.Key && !item.Key.endsWith("/")) {
            keys.push(item.Key);
          }
        });
      }
      continuationToken = output.NextContinuationToken;
    } while (continuationToken);
    return keys;
  }
  createLocationSpec(key) {
    return {
      type: "url",
      target: this.createObjectUrl(key),
      presence: "required"
    };
  }
  createObjectUrl(key) {
    return new URL(key, this.endpoint).href;
  }
}

exports.AwsS3EntityProvider = AwsS3EntityProvider;
//# sourceMappingURL=AwsS3EntityProvider.cjs.js.map
